#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################

import re,os,sys,shutil,multiprocessing,glob
import BlastTIRAnalyse, ucfAur,BlastAnalyse,ExtractSubSeq,multimafft
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from collections import Counter

def autonomous(input, genome): #BilanTIR.fa file as input
    # create dir 'Auto'
    try: os.mkdir(os.path.dirname(input)+'/Auto')	
    except: print ''
    newlocfile=os.path.dirname(input)+'/Auto/'+os.path.basename(input)
    shutil.copyfile(input, newlocfile)
    seqfile=newlocfile+'.blastn.bpia.seq'

    # blast for paired ITR
    res=BlastTIRAnalyse.blastTIR(newlocfile, genome)
    bilanAutonomous=open(os.path.dirname(input)+'/BilanAutonomous','w')
    
    if res==None: 
        return None

    # cluster pairs and filter
    print '\t>>> Clustering...'
    os.system('uclust --quiet --sort %s --output %s.sorted'%(seqfile,seqfile))
    os.system('uclust --quiet --cluster %s.sorted --seedsout %s.uniq --uc %s.uc --rev --id 0.70 '%(seqfile,seqfile,seqfile))
    clust=len(list(SeqIO.parse(seqfile+'.uniq','fasta')))
    # Calculate number of autonomous sequence per MITE cluster
    bilanAutonomous=open(os.path.dirname(input)+'/BilanAutonomous','w')
    Records=list(SeqIO.parse(newlocfile+'.blastn.bpia.seq','fasta'))
    listAutonomous=[seq.id.split('_')[0] for seq in Records]
    dictAutonomous=dict(Counter(listAutonomous))
    
    # Write BilanAutonomous file
    for i in dictAutonomous:
        bilanAutonomous.write(i.rstrip()+'\t'+str(dictAutonomous[i])+'\n')
    bilanAutonomous.close()
    print '\t',len(dictAutonomous), 'TIR-clusters have longer elements'
    print '\t',clust, 'USEARCH-clusters have longer elements'
    return len(dictAutonomous),clust
    
# not yet functional
def incomplete(input, genome): # BilanTIR file
    newlocfile=os.path.dirname(input)+'/Auto/'+os.path.basename(input)
    seqfile=newlocfile+'.blastn.bpia.seq'    

    # Blast unique copie to find complete and inclomplete copies # blast and blast+ needed
    if os.path.isfile(genome+'.nhr'):pass
    else: 
        os.system('formatdb -i %s -p F'%genome)
        #os.system('makeblastdb -in %s'%genome)
    os.system('blastn -query %s.uniq -out %s.autonomous.blastn -db %s -evalue 1e-5 -outfmt 6'%(seqfile,newlocfile,genome))
    gap='1000'
    print    
    # Analysing BLAST result, and extracting sequences in a row
    BlastAnalyse.count(newlocfile+'.autonomous.blastn', gap, '3')
    seqfile2=ExtractSubSeq.SubSeqGo(newlocfile+'.autonomous.blastn.pba',genome,'60') 
    
    #
    print '\t>>> Clustering...'
    os.system('uclust --quiet --sort %s --output %s.sorted'%(seqfile2,seqfile2))
    os.system('uclust --quiet --cluster %s.sorted --seedsout %s.uniq --uc %s.uc --rev --id 0.8'%(seqfile2,seqfile2,seqfile2))
    
    # Filtering and Aligning copies in clusters
    seqfile=ucfAur.filtre(seqfile2+'.uc',seqfile2,os.path.dirname(input)+'/Auto/Clusters',0)
    if seqfile[1]==0: return None
    nbcpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
    nbcpu.map(eval('multimafft.mafftor'), seqfile[0]) #'*' list
    return seqfile
    
    
# not yet functional
def exhaustive(input, genome): # Cons file
    #create dir 'full'
    try: 
        os.mkdir(os.path.dirname(input)+'/AllMITE')
    except: print ''
    try:
        os.mkdir(os.path.dirname(input)+'/AllMITE/Clusters')	
    except: print ''
    newlocfile=os.path.dirname(input)+'/AllMITE/'+os.path.basename(input)
    shutil.copyfile(input, newlocfile)
    
    if os.path.isfile(genome+'.nhr'):pass
    else: 
        os.system('formatdb -i %s -p F'%genome)
        #os.system('makeblastdb -in %s'%genome)
    os.system('blastn -query %s -out %s.blastn -db %s -evalue 1e-5 -outfmt 6'%(newlocfile,newlocfile,genome))
    gap='200'
    
    BlastAnalyse2.count(newlocfile+'.blastn', gap, '3')
    
    seqfile2=ExtractSubSeq.SubSeqGoSeparate(newlocfile+'.blastn.pba',genome,'60') 
    seqfile=glob.glob(os.path.dirname(input)+'/AllMITE/Clusters/*.seqfl')
    nbcpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
    nbcpu.map(eval('multimafft.mafftor'), seqfile) #'*' list
    
if __name__ == '__main__' : 
    #exhaustive(sys.argv[1],sys.argv[2])
    autonomous(sys.argv[1],sys.argv[2])