#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################


import re,os,sys, shlex, subprocess
import ExtractSubSeq
global limite
global gap
"""
From a BLAST result (tab) from TIR query : reassociate TIR pairs distant from less than the max distance 
Input: a BLAST result (tab) from TIR query
Output: 
- a file with paired TIR (pba2 type) (bpia)
- a file with solo TIR (bsia)
"""

def ask():
	global gap,limite
	input=raw_input("Please provide full path to TIR BLAST result file ? ")
	genome=raw_input("Please provide the genome database used ? ")
	count(input)
	gap=raw_input('Maximum distance between 2 TIRs [5000]? ')
	if gap=='':gap='5000'	
	limite=raw_input('Length of flanking sequences to extract? [60] ')
	if limite=='':limite='60'
	
###################################################################################
##################		 RUN TIR BLAST ON GENOME    ###############################
###################################################################################
def blastTIR(query, genome): # need blast+ or blastall installed
    dirbase=os.path.dirname(query).split('/Auto')[0]     
    word=int(open(dirbase+'/options').readlines()[5].split('\t')[1])
    try: os.system('makeblastdb -i %s '%genome)# input genome
    except: os.system('formatdb -i %s -p F'%genome)
    try: os.system('blastn -task blastn-short -query %s -subject %s -out %s.blastn -word_size %s -outfmt 6 -dust no -evalue 50 '%(query,genome,query,word))
    except: os.system('blastall -p blastn -i %s -d %s -o %s.blastn -m 8 -e 10'%(query, genome,query))
    ret=count(query+'.blastn', genome) 
    return ret
    
################################################################################
###########		   COUNT TIR PAIRS AND SOLO TIR		 #######################
################################################################################
def count(input, genome):
	limite='60'
	gap='5000'
	dico_pair={} #limite de l'homologie (tuple)
	dico_solo={}  # position moyenne de l'homologie
	dico_scaffold={}

	dico_pair.clear()
	dico_solo.clear() 
	
	line=open(input).readlines()

	
	# each hsp is identified by query and subject, and characterized by its subject coordinates
	# Pour chaque ligne de blast... mise en dico[Element_Contig]=liste de tuple de coordonees
	for i,x in enumerate(line): # 
		#print x.split('\t')[1]+'&'+x.split('\t')[0]
		if x.split('\t')[1]+'&'+x.split('\t')[0] not in dico_scaffold.keys():
			dico_scaffold[x.split('\t')[1]+'&'+x.split('\t')[0]]=[(int(x.split('\t')[8]),int(x.split('\t')[9]))]
		else:
			dico_scaffold[x.split('\t')[1]+'&'+x.split('\t')[0]].append((int(x.split('\t')[8]),int(x.split('\t')[9])))
	
	# For each contig, analysis of orientation and order relative fo previous hsp
	for tuplist in dico_scaffold:
		pair3=()
		pair5=()
		#print 
		#print tuplist, ':', dico_scaffold[tuplist]
		dico_scaffold[tuplist].sort()
		start,end=zip(*dico_scaffold[tuplist])
		lstart,lend=list(start),list(end)
		
		while len(lstart)>0:
			#print min(lstart),(lend)[lstart.index(min(lstart))],
			# si 1ere ITR  dans mauvais sens ... 
			if min(lstart) > lend[lstart.index(min(lstart))]: 
				if tuplist not in dico_solo.keys(): 
					# --> SOLO  dans dico_solo
					dico_solo[tuplist]=[(min(lstart),lend[lstart.index(min(lstart))])]
 					#print 'bad orientation ... in dico_solo (0)',dico_solo[tuplist]
				else: 
					# --> SOLO  dans dico_solo
					dico_solo[tuplist].append((min(lstart),lend[lstart.index(min(lstart))]))
					#print 'in dico_solo (1)', dico_solo[tuplist]
					
				# Suppression dans les liste(tuple)
				lend.pop(0)
				lstart.pop(0)
				#print len(lstart)				
				if len(lstart)==0:break
				
			# si 1ere ITR  dans bon sens ... 			
			else :
				#print 'good first ITR',
				pair5=(min(lstart),lend[lstart.index(min(lstart))])

				lend.pop(0)
				lstart.pop(0)
				
				# mais pas d'autre ITR
				if len(lstart)==0:
					#print 'but no 2nd ITR!... in dico_solo (2)'
					# --> SOLO  puis BREAK dans dico_solo
					if tuplist not in dico_solo.keys(): dico_solo[tuplist]=[pair5]
					else: dico_solo[tuplist].append(pair5)
					#print dico_solo[tuplist]
					break
				else :
					#print min(lstart),lend[lstart.index(min(lstart))],
					# et 2nd ITR en sens inverse (attendu)
					if min(lstart) > (lend)[lstart.index(min(lstart))]: 
						pair3=(min(lstart),lend[lstart.index(min(lstart))])
						if tuplist not in dico_pair.keys():dico_pair[tuplist]=[(pair5,pair3)]
						else:dico_pair[tuplist].append((pair5,pair3))	
						lend.pop(0)
						lstart.pop(0)
						#print  'in dico_pair'
						#print dico_pair[tuplist]
						
					# Si 2nd ITR dans sens 1
					else :	
						if tuplist not in dico_solo.keys(): 
							dico_solo[tuplist]=[pair5]
						else: 						
							dico_solo[tuplist].append(pair5)
						#print 'but not good 2nd ITR'
						#print pair5,'in dico_solo (3)'
						#print dico_solo[tuplist]

					#print len(lstart)
	print '\tThere are ',len(dico_pair), "complete potential autonomous copies "
	print '\tThere are ',len(dico_solo), "solo TIRs"
	# write files with coordinates
	bsia=open(input+'.bsia','w') # solo TIR
	bpia=open(input+'.bpia','w') # Pair TIRs
	biaf=open(input+'.bsiaf','w') # solo TIRs and flanking
	
	
	for z in dico_pair:
		for pair in dico_pair[z]:
			if pair[1][0]-pair[0][0] <= int(gap)and pair[1][0]-pair[0][0]>1000:
				bpia.write(z.split('&')[0]+'\t'+z.split('&')[1]+'_'+z.split('&')[0]+'\t')
				bpia.write(str(pair[0][0]-1)+'\t'+str(pair[1][0]-1)+'\t'+str(pair[1][0]-pair[0][0])+'\t')
				bpia.write(str(abs(pair[0][1]-pair[0][0]))+'\t'+str(abs(pair[1][1]-pair[1][0]))+'\t'+z.split('&')[1]+'\n')

	for y in dico_solo:
		for solo in dico_solo[y]:
			maxi= max(solo[1],solo[0])+int(limite)
			mini= min(solo[1],solo[0])-int(limite)
			bsia.write(y+'\t'+str(solo[0])+'\t'+str(solo[1])+'\n')
			biaf.write(y+'\t'+str(mini)+'\t'+str(maxi)+'\n')


	bsia.close()
	biaf.close()
	bpia.close()
	dico_solo.clear()
	
	ret=ExtractSubSeq.SubSeqGo(input+'.bpia', genome,limite)
	if ret==None:return None
	else:  
	    print '\t', len(ret), 'sequences kept'
	    return ret

if __name__ == '__main__':
    blastTIR(sys.argv[1], sys.argv[2])