#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################

import re,os,sys
from Bio.Alphabet import IUPAC
from Bio.Seq import Seq
from Bio import SeqIO
import MakeFasta
from glob import iglob
from Bio.SeqRecord import SeqRecord

"""
Ce programme lit une liste de numeros d'accession_nom, et des coordonnees, va chercher les sequences 
correspondantes dans un fichier fasta, et cree un nouveau fichier fasta, ajoute les falnquantes
- fasta file with extracted sequences (.seq)
- fasta file with extracted sequences and flanking (.seqfl)
"""

def ask(*args):

    print args
    if len(args)==0: nom=raw_input('ExtractSubSeq >> nom du fichier contenant les accessions ? [.pba ou pba2 ou Allinfo]: ').strip()
    else: nom=args[0]
    if len(args)==2: fasta=args[1]
    else: genome=raw_input('ExtractSubSeq >> nom du(es) fichier(s) contenant les sequences fasta? (*)/[GB]): ').strip()
    #save=raw_input('ExtractSubSeq >> nom du fichier a sauvegarder? [.seq]: ')
    #if save=='':

    flank=raw_input('ExtractSubSeq >> longueur des flankantes? [100] : ')
    if flank=='':flank='100'  
    print "Running ExtractSubSeq..."
    launch(nom,genome,flank) 
    
def launch(nom,genome,flank) :
    save=nom+'.seq'
    print save
    # Suppression fichiers de sortie existant seq et seqfl
    if os.path.isfile(save): os.remove(save)
    if os.path.isfile(save+'fl'): os.remove(save+'fl')

    if '*' in genome:
        for bank in iglob(genome):
            SubSeqGoSeparate(nom,bank,flank)
            print bank
    elif '*' in nom:
        for name in iglob(nom):
            SubSeqGoSeparate(name,bank,flank)
            print name
    else:  SubSeqGoSeparate(nom,genome,flank)
    return nom+'.seq'

    
def SubSeqGo(nom,genome,flank) :

    ############ OUVERTURE ET LECTURE DES FICHIERS        #########################
    
    # ouverture et lecture fichier accession pba --> liste
    fileAcc=open(nom).readlines() 
    ListContig=set([x.split('\t')[0] for x in fileAcc])
    print  "\tThere are ", len(fileAcc), "sequences to extract"
    print  "\tThere are ", len(ListContig), "different accessions(contigs)"
    if len(fileAcc)==0: 
        print "no sequences to extract"
        return None

    # choix database
    if genome=='GB' or genome=='' :
        pass
        
    else: 
        fileseq=ExtractSeq(nom,genome,fileAcc,flank)
        if len(fileseq[0]) ==0: 
            print 'return extract seq empty', fileseq[0]
            return None
        SeqIO.write(fileseq[0],nom+'.seq','fasta')
        SeqIO.write(fileseq[1],nom+'.seqfl','fasta')  
    return fileseq[0]
        
def SubSeqGoSeparate(nom,genome,flank) :
    print '\n>>> Calculating sequences , contig and cluster number'

    ############ OUVERTURE ET LECTURE DES FICHIERS        #########################
    
    # ouverture et lecture fichier accession pba --> liste
    fileAcc=open(nom).readlines() 
    ListContig=set([x.split('\t')[0] for x in fileAcc])
    dicoCluster={}
    fileseq=[]
    for x in fileAcc:
        if x.split('\t')[-1].rstrip() in dicoCluster.keys():dicoCluster[x.split('\t')[-1].rstrip()].append(x)
        else: dicoCluster[x.split('\t')[-1].rstrip()]=[x]
    print  "\tThere are ", len(fileAcc), "sequences to extract"
    print  "\tThere are ", len(ListContig), "different accessions(contigs)"
    print  "\tThere are ", len(dicoCluster), "different clsuters"

    print '\n>>> Extracting sequences for', len(dicoCluster), 'clusters'        
    for clust in dicoCluster:
        fileseq=ExtractSeq(clust,genome,dicoCluster[clust],flank)
        SeqIO.write(fileseq[0],os.path.dirname(nom)+'/Clusters/'+clust+'.seq','fasta')
        SeqIO.write(fileseq[1],os.path.dirname(nom)+'/Clusters/'+clust+'.seqfl','fasta')
    return os.path.dirname(nom)+'/Clusters/'+clust+'.seq'
    
def ExtractSeq(nom,genome,fileAcc,flank):

    # ouverture fichier de sortie
    listseq=[]
    listseqf=[]
    #sauve=open(nom+'.seq','a')
    #sauvef=open(nom+'.seqfl','a')
    listAcc=[hit.split('\t')[0] for hit in fileAcc]
    # ouverture fichier genome
    record_iterator = SeqIO.parse(genome,'fasta')
    while True:
        try: supercontig = record_iterator.next()    
        except StopIteration: 
            break
        b=0
        for i,hit in enumerate(fileAcc):

            #print supercontig.id, hit
            if hit =='\n' :pass               
            elif supercontig.id == hit.split('\t')[0]:
                name=hit.split('\t')[1]
                mini=min(int(hit.split('\t')[2]),int(hit.split('\t')[3]))
                minif=max(mini-int(flank),0)
                maxi=max(int(hit.split('\t')[2]),int(hit.split('\t')[3]))
                maxif=maxi+int(flank)             
                if int(hit.split('\t')[2])>int(hit.split('\t')[3]) or '(-)' in hit.split('\t')[1]: # SENS -
                    laSeq=supercontig.seq[mini:maxi+1].tomutable()
                    laSeq.reverse_complement()
                    laSeqFinale=laSeq.tostring()
                    laSeqf=supercontig.seq[minif:maxif+1].tomutable()
                    laSeqf.reverse_complement()
                    laSeqFinalef=laSeqf.tostring()                 
                else:
                    laSeqFinale=supercontig.seq[mini:maxi+1].tostring() # SENS +
                    laSeqFinalef=supercontig.seq[minif:maxif+1].tostring() # SENS +
                if hit.split('\t')[0]==hit.split('\t')[1]: ns='_'+str(b) # If no seq name
                elif len(set(listAcc))<len(fileAcc):ns='_'+str(b) # if redundant seq name
                else:ns=''

                #sauve.write('>'+name+'_'+hit.split('\t')[2]+'_'+hit.split('\t')[3]+'\n'+laSeqFinale+'\n')                
                #sauvef.write('>'+name+'_'+hit.split('\t')[2]+'_'+hit.split('\t')[3]+'\n'+laSeqFinalef+'\n')             
                if 'NNNNNNNNNNNN' in laSeqFinale: pass
                else:
                  listseq.append(SeqRecord(Seq(laSeqFinale),id=name+'_'+hit.split('\t')[2]+'_'+hit.split('\t')[3],description=''))
                  listseqf.append(SeqRecord(Seq(laSeqFinalef),id=name+'_'+hit.split('\t')[2]+'_'+hit.split('\t')[3],description=''))
                  b=b+1
                fileAcc[i]='\n'             


    #sauve.close()  
    #sauvef.close()
    if flank=='0': os.remove(nom+'.seqfl')
    #return nom+'.seq'
    return listseq,listseqf,b
    
if __name__ == '__main__':
    if len(sys.argv)==1: ask()
    elif len(sys.argv)==4: launch(sys.argv[1],sys.argv[2],sys.argv[3])