#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################
from Bio import SeqIO
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Align import AlignInfo
from Bio.Alphabet import IUPAC
from Bio.Align import MultipleSeqAlignment
from StringIO import StringIO
import os, glob, shutil

def createfiles(file):
    # reverse complement # .ifl.afa --> .ifl.afa.rc
    ReverseComplementCluster(file)
    os.system('muscle -quiet -profile -in1 %s -in2 %s -out %s'%(file,file+'.rc',os.path.splitext(file)[0]+'.faa')) # .ifl.afa.rc --> .ifl.faa
    # implement mafft7
    # os.system('mafft-profile  %s  %s > %s'%(file,file+'.rc',os.path.splitext(file)[0]+'.faa')) # .ifl.afa.rc --> .ifl.faa
    os.remove(file+'.rc')


def AnalyseCluster(clust):
    clustbase=clust.split('.ifl.faa')[0]
    bound=fb(clust) # return start and end of MITE TIRs in the ifl.faa file (distal 5'TIR, proximal 5'TIR, proximal 3'TIR, distal 3'TIR, total length)
    if bound[3]>bound[0] and bound[4]-bound[3]>10 and bound[0]>28:
        cleanclust=PushGap(clust, bound) # return clean .ifl.faa file as .ifl.faa  (flanking gaps pushed toward the ends)
        bound=fb(cleanclust) # return NEW start and end of MITE from the .ifl.faa file
    if bound[3]>bound[0] and bound[4]-bound[3]>10 and bound[0]>28:  
        TrimTaxa(clust, clustbase) # remove rc sequences (deduplicate) .ifl.faa --> .fl.fa
        TrimFlanking(clustbase, bound) # Trim flanking in clustbase.fl.fa --> .afa
        SaveFlanking(clustbase, bound) # Save flanking in clustbase.fl.fa --> .fl53
        os.remove(clustbase+'.fl.fa')
        Squeeze(clustbase+'.afa')
        #os.system('seqmagick mogrify --squeeze %s'%(clustbase+'.afa')) # Trim gap-only column in clustbase.afa  
        # return a string containing Cluster name, start and end of MITE, length of TIRs
        return os.path.basename(clustbase)+'\t'+str(bound[0])+'\t'+str(bound[4]-bound[3])+'\t'+str(bound[1]-bound[0])+'\t'+str(bound[3]-bound[2]+2)+'\n'
    else: 
        Filetomove=clustbase+'*'
        for fileb in glob.glob(Filetomove): 
            new=os.path.basename(fileb)
            os.rename(fileb,os.path.dirname(fileb)+'Exclu2/'+new)
        return 1
    	        
def fb(clust):

	alignment = AlignIO.read(clust, "fasta")
	mini5,mini3=0,0
	maxi5,maxi3=0,0
	stretch5,stretch3=0,0
	stretchend5,stretchend3=0,0
	gap5,gap3=0,0
	ITR=False
	#print clust
	# 5' treatment
	for nt5 in range(alignment.get_alignment_length()):
	    
	    if stretch5>=10 and nt5-maxi5==5:
	        #print nt5, 'break'
	        break
	    else:
	        column=alignment[:,nt5] # All sequences, position nt
	        col=[column.count('A'),column.count('C'),column.count('G'),column.count('T')]
	        
	        if sum(col)==0 or (column.count('-')>len(alignment)/2 and stretch5>3):pass # if gap only, or gaps everywhere except one sequence
	        elif float(max(col))/sum(col)>=0.8 and  len(alignment)-sum(col)<2 : # id if 3/4 and 1 gap at max
	            if stretch5>10 and stretchend5<4: pass# 
	            else: stretchend5=0
	                
	            gap5=0
	            stretch5+=1
	            ITR=True
	        else: # if diff
	            if sum(col)== len(alignment)/2 and stretch5>10: gap5+=1 # if diff after 3 id, decrease id, increase diff
	            #if ITR==False:stretch5-=1
	            if stretch5>10: pass # if diff after 10 id, do nothing to id, increase diff
	            elif stretch5>3: stretch5-=1 # if diff after 3 id, decrease id, increase diff
	            #elif ITR==True: pass
	            else: stretch5=0 # if diff after less than 3 id, reset id, increase diff
	            stretchend5+=1
	            ITR=False  
	        #print nt5, stretch5,stretchend5, gap5
	    if stretch5==1 and stretchend5==0: 
	        #print nt5
	        mini5=nt5        
	    if stretch5>10 and stretchend5==1: 
	        #print nt5
	        maxi5=nt5
	    #print nt5, alignment[1,nt5], ITR, '\t',stretch5, stretchend5,'\t\t', mini5, maxi5
	#print mini5+1,maxi5+1,alignment.get_alignment_length()-(maxi5-1),alignment.get_alignment_length()-(mini5)
	return mini5+1,maxi5+1,alignment.get_alignment_length()-(maxi5-2),alignment.get_alignment_length()-(mini5), alignment.get_alignment_length()
	
	
def ReverseComplementCluster(file): # reverse complement alignment 
	newalign= [rec.reverse_complement(id=rec.id, description = '') for rec in SeqIO.parse(file, "fasta")]
	SeqIO.write(newalign, file+'.rc', "fasta")
	#return clust

def PushGap(clust, bound):
	alignment = AlignIO.read(clust, "fasta")
	newalign=[]
	for seq in alignment:
	    n5=seq[:bound[0]].seq.count('-')
	    n3=seq[bound[3]:].seq.count('-')
	    newseq=SeqRecord(Seq('-'*n5+str(seq.seq[:bound[0]]).replace('-','')+ str(seq.seq[bound[0]:bound[3]]) +str(seq.seq[bound[3]:]).replace('-','') +'-'*n3),id=seq.id, description='')
	    newalign.append(newseq)
	shutil.copyfile(clust,clust+'0')
	SeqIO.write(newalign, clust, "fasta")
	return clust

def TrimTaxa(clust, clustbase): # deduplicate taxa from .ifl.fa --> .fl.fa
	dicoUniq={}
	newalign=[]
	alignment = AlignIO.read(clustbase+'.ifl.faa', "fasta")
	for seq in alignment:
	    if seq.id in dicoUniq.keys():pass
	    else: 
	        dicoUniq[seq.id]=seq
	        newalign.append(seq)
	SeqIO.write(newalign, clustbase+'.fl.fa', "fasta")
	#return clust

def TrimFlanking(clustbase, bound): # Save aligment of flanking sequences
	alignment = AlignIO.read(clustbase+'.fl.fa', "fasta")
	align=alignment[:,bound[0]-1:bound[3]]
	SeqIO.write(align, clustbase+'.afa', "fasta")
	return clustbase+'.afa'

def SaveFlanking(clustbase, bound): # Save aligment of 30 bp flanking sequences
	alignment = AlignIO.read(clustbase+'.fl.fa', "fasta")
	flankalign=alignment[:,bound[0]+1-30:bound[0]+2]+alignment[:,bound[3]-3:bound[3]-2+30]
	SeqIO.write(flankalign, clustbase+'.fl53', "fasta")
	
def Squeeze(clustbaseafa): # remove column made of gaps only
	alignment = AlignIO.read(clustbaseafa, "fasta")
	for nt in range(alignment.get_alignment_length()):
	    if nt==0:
	        new_alignment=alignment[:,nt:nt+1]
	    column=alignment[:,nt] # All sequences, position nt
	    if column.count('-')==len(alignment): pass
	    elif nt!=0: 
	        new_alignment += alignment[:,nt:nt+1]        
	SeqIO.write(new_alignment, clustbaseafa, "fasta")


