#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################
from Bio import SeqIO
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Align import AlignInfo
from Bio.Alphabet import IUPAC
from Bio.Align import MultipleSeqAlignment
from StringIO import StringIO
import os

def analyseTSD(file): #.fl53
        alignment = AlignIO.read(file, "fasta") # list of sequence # ouvrir comme alignment
        TSDList=[]
        
        for seq in alignment:
            #print os.path.basename(file), seq.id
            motif5=str(seq.seq[0:alignment.get_alignment_length()/2:])
            motif3=str(seq.seq[alignment.get_alignment_length()/2:])
            TA=False
            TSD=True
            maxscore=0
            maxi=len(motif5)
            maxn=0
            max5=0
            mini5=0
            mini3=0
            BetterTSD=''
            BetterTSDTA=''
            for i in reversed(range(len(motif5))): # each sequence comparison
                #print i, seq.id
                #print motif5[i:len(motif5)]
                #print motif3[:len(motif5)-i]
                TSD5=''
                TSD3=''
                n=0
                score=0

                while i+n<len(motif5): # for each aligned nt within len of motif5
                    #print 'motif5[',i+n,']<==> motif3[',n,']', motif5[i+n],motif3[n]
                    if motif3[n] == motif5[i+n]: # ID
                        score+=1
                        TSD5+=motif5[i+n]
                        TSD3+=motif3[n] 

                    else: # diff : reset score
                        score=0
                        TSD5=''
                        TSD3=''
                    

                   # if score==maxscore and 'TA' in TSD5 and 'TA' not in BetterTSD:
                        #score+=1
                    if 'TA' in TSD5 and BetterTSDTA=='':
                        valuesTA=(i+n-score+1,i+n+1,n-score+1,n+1,i-1)
                        BetterTSDTA='TA'#motif5[valuesTA[0]:valuesTA[1]]
                        #print 'TA', valuesTA
                    if score > maxscore and abs(n-score+1-(len(motif5)-(i+n+1)))<=2: # new id after id, same distance from the end
                        #print 'maxscore',maxscore, '< score',score
                        #print 'maxi = ',i
                        #print 'maxn = ',n
                        #print 'max5 = ',i+n
                        
                        maxi=i # decalage du meilleur score
                        maxn=n
                        max5=i+n
                            #print 'new maxscore',score
                        #print 'found better score','i ==',i,'n =',n
                        BetterTSD=TSD5
                        if TSD5!=TSD3: TSD=False
                        else:TSD=True
                        #print TSD5,TSD,score
                    #print TSD3 
                    # end of the nt comparison
                    maxscore=max(maxscore,score)
                    values=(max5-maxscore+1,max5+1,maxn-maxscore+1,maxn+1,maxi-1)
                    n+=1

                   
                #print BetterTSD
                if i>=len(motif5)-4 and BetterTSDTA=='TA': # Check for TA for each 4 first comparisons
                    #print 'YOP', i, len(motif5)-4, BetterTSD
                    #print TSD5,TSD3
                    TA=True
                    #print i
                    #valuesTA=(max5-maxscore+1,max5+1,maxn-maxscore+1,maxn+1,maxi-1)
                    #BetterTSDTA='TA'#motif5[valuesTA[0]:valuesTA[1]]
                if TSD5!=TSD3:
                    TSD=False
                    
            #print maxscore, maxi, max5, maxn
            #print maxn-maxscore+1,len(motif5)-(max5+1)
            if TA==True: 
                BetterTSD=BetterTSDTA
                values=valuesTA
#                 print 'found TA'
#                 print BetterTSDTA
#                 print motif5[valuesTA[0]:valuesTA[1]], valuesTA[0:2]
#                 print motif3[valuesTA[2]:valuesTA[3]], valuesTA[2:4]
#                 print motif5
#                 print ' '*(valuesTA[4]),motif3                
#                 print valuesTA
#             print BetterTSD
#             print motif5[values[0]:values[1]], values[0:2]
#             print motif3[values[2]:values[3]], values[2:4]
#             print motif5
#             print ' '*(values[4]),motif3
#             print values
            #print maxscore, TSD, maxi,maxn,maxn-maxscore+1,maxn+1
            #print len(motif5)-maxi ,maxscore+6    
            TSDList.append(BetterTSD)
        lenlist=sorted([len(i) for i in TSDList],reverse=True)
        #print TSDList, lenlist
        TAlist=[i for i in TSDList if i=='TA']
        #print
        #print os.path.basename(file)
        length=max(lenlist, key=lenlist.count)
        #print 'length : ', length,'\tConfidence : ', round(float(lenlist.count(max(lenlist)))/len(TSDList),2), '('+str(lenlist.count(max(lenlist)))+'/'+str(len(TSDList))+')'
        if len(TAlist)==0: TAinfos='\t\t\t'
        else: TAinfos='\tTA\t'+str(len(TAlist))+'\t'+str(len(TSDList))
        Bilan = os.path.basename(file).split('.')[0]+ '\t' + str(length) +'\t'+str(lenlist.count(max(lenlist)))+'\t'+str(len(TSDList))+TAinfos
        return Bilan