#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################

from optparse import OptionParser
import time, glob
import re, os, sys, shutil,logging
import multiprocessing ,subprocess, shlex
from Bio import SeqIO
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Align import AlignInfo
from Bio.Alphabet import IUPAC
from StringIO import StringIO
from Bio.Alphabet import generic_dna
from Bio.Align import MultipleSeqAlignment


def TooMuchGap(clust): # an alignment of independant sequences without flanking .afa
    dirbase=os.path.dirname(clust).split('/Clusters')[0]
    minsize=int(open(dirbase+'/options').readlines()[3].split('\t')[1])
    clustbase=os.path.splitext(clust)[0]

    ExcludedSeq=[]
    Length={}
    TotalColumnWithGap=0
    ListSeq=[]
    alignment = AlignIO.read(clust, "fasta")
    # Number of gap per sequence
    for seq in alignment: Length[seq.id]=alignment.get_alignment_length()-seq.seq.count('-')
    # Average length of sequences
    Moyenne=float(sum(Length.values()))/len(Length)
    
    # number of gap in each sequence must be less than 25 % of the mean length of sequences
    for seq in alignment: 
        if seq.seq.count('-') > Moyenne/5 : ExcludedSeq.append(seq.id)
        if len(str(seq.seq).replace('-','')) > (Moyenne + Moyenne/4):ExcludedSeq.append(seq.id)
        else: ListSeq.append(seq)
    
    # new cluster file if more than 1 sequence
    newalign=MultipleSeqAlignment(ListSeq)
    if len(ListSeq)>1: 
        AlignIO.write(newalign,clust, "fasta")

        alignment2 = AlignIO.read(clust, "fasta")
        for nt in range(alignment2.get_alignment_length()): # number of column with gap
            if alignment2[:,nt].count('-') == len(alignment2):pass
            if '-' in alignment2[:,nt] : TotalColumnWithGap+=1
            
    if len(ExcludedSeq) >= float(len(alignment)/2) or TotalColumnWithGap > float(alignment2.get_alignment_length())/4:
        #print Moyenne, len(ExcludedSeq)
        BadCluster(clust,'toomanygap')
    elif alignment.get_alignment_length() < minsize: BadCluster(clust,'tooshort')
    else: 
        return 1  #    else: BadCluster(clust)

    #return length alignment and number of sequence

def BadCluster(clust,message):
    clustbase=os.path.splitext(clust)[0]
    print "rejecting", os.path.basename(clustbase), message
    for file in glob.glob(clustbase+'*'):
        os.rename(file, os.path.dirname(file)+'Exclu3/'+os.path.basename(file))
