#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################

import re, sys, shutil, os, subprocess, shlex, time, glob
from Bio import SeqIO


def splitgenome(genometosplit,splitsize,dirsuffix):

	listsubcontig=[]
	print "\n>>> processing file in subcontigs"
	t1=time.time()
	# create and process subfiles
	listsubcontig=[]
	for seq_record in SeqIO.parse(genometosplit, "fasta"):
	    n=0
	    #print 'processing ', seq_record.id
	    while n<=len(seq_record)/splitsize:
	        newfile=genometosplit + '.dir/BLAST/'+ seq_record.id+'_'+str(n)
	        subcontig=open(newfile,'w')
	        try: subcontig.write('>'+seq_record.id+'_'+str(n)+'\n'+seq_record.seq.tostring()[n*splitsize:(n+1)*splitsize+splitsize/20])
	        except:subcontig.write('>'+seq_record.id+'_'+str(n)+'\n'+seq_record.seq.tostring()[n*splitsize:])
	        subcontig.close()
	        listsubcontig.append(newfile)
	        n+=1
	return listsubcontig
	

def concat(genometosplit, prog,splitsize): # concat in one file (BLAST.out) 
	print '\n>>> concatenating result'
	quest = genometosplit+'.dir/'+prog+'/*.'+prog+'out'
	all = genometosplit+'.dir/'+prog+'/'+prog+'.out'
	tothsp=0
	dicontig={}
	resultout=open(all,'w')
	
	for i in glob.glob(quest): #filter1(i)
		result=open(i).readlines()
		nb=int(os.path.basename(i).split('_')[-1].split('.'+prog+'out')[0]) # Slice number
		supercontig='_'.join(os.path.basename(i).split('_')[:-1])
		if supercontig not in dicontig.keys() :dicontig[supercontig]=[] #new supercontig dict
		for hsp in result:
			if hsp=='' or hsp[0] =='#'or hsp==' ': pass
			else:
				hsp=hsp.replace(hsp.split('\t')[0],supercontig)
				for x in range(6,10): # Correct for rigth positions
					hsp=hsp.replace(hsp.split('\t')[x],str(nb*int(splitsize)+int(hsp.split('\t')[x])))
				if hsp.split('\t')[6]==hsp.split('\t')[8] and hsp.split('\t')[7]==hsp.split('\t')[9]:pass#self
				else: 
				    if hsp.split('\t') not in dicontig[supercontig]: # new hsp
				        resultout.write(hsp)
				        tothsp+=1
		os.remove(i)
	print tothsp
	return resultout

def IRFBlastize(IRFoutfile): #.dat --> IRF.out (Blast format)

	file = open(IRFoutfile).read().split('Sequence: ')[1:] # split result by contig
	print len(file), ' contigs'
	resultout = open(os.path.dirname(IRFoutfile) +'/IRF.out' ,'w')
	palindromef=[] # useless
	totpal=0
	for contig in file: 
	    name=contig.split('\n')[0]
	    palindrome=contig.split('\n')[7:-3]
	    totpal+=len(palindrome)
	    for i,line in enumerate(palindrome):
	        listitem=line.split(' ') # list info for each hit
	        lenTIR=max(int(listitem[2]),int(listitem[5])) # calculate TIR length
	        if float(listitem[12])>90 or float(listitem[13])>90:pass # filter out TIRs with low complexity
	        else:
	            # query, subject, % match, TIR length, nb of mismatch, nb of indel, query start, query end, subject start, subject end
	            resultout.write(name + '\t' + name +'\t'+str(round(float(listitem[7]),2))+'\t'+str(lenTIR)+'\t'
	            +str(int(listitem[2])-int(lenTIR*float(listitem[7])/100))+'\t'+str(lenTIR*float(listitem[8])/100)+'\t'
	            +line.split(' ')[0]+'\t'+line.split(' ')[1]+'\t'+line.split(' ')[4]+'\t'+line.split(' ')[3]+'\t\n')
	resultout.close()

