#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van

#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################


from optparse import OptionParser
import time, glob,itertools
import re, os, sys, shutil,logging
import multiprocessing, subprocess, shlex
from Bio import SeqIO
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Align import AlignInfo
from Bio.Alphabet import IUPAC
from StringIO import StringIO
from Bio.Alphabet import generic_dna
from Bio.Align import MultipleSeqAlignment
import Mask, ucfAur, multimafft,filtreIR, ConsAur,remove_duplicate, MakeFasta, FindBoundaries,RejectClusters,FindTSD
import Autonomous,SplitGenome
import Check
global dirsuffix
global M
global tinit    
dirsuffix='.dir'
M ='M'
nbcpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
tinit=time.time()

def ask():
    input=raw_input('Which genome (Full path for genome fasta file) ?').rstrip() # Ask for Genome file
    program=raw_input('What progrram BLAST/[IRF] ?').rstrip() # Ask for Program to find TIRs
    goon(input, program)

#####################################################################
##########  -1-     MASK GENOME   
#####################################################################
def goon(input, prog):
    global dirsuffix,M
    
    try: os.mkdir(input + M + dirsuffix) 
    except:print str(input + M + dirsuffix), " already exists" 
    try: os.mkdir(input + M + dirsuffix + '/' +options.program) 
    except:print str(input + M +dirsuffix + '/' +options.program), " already exists"    
    
    # create directory PAL, BLAST or IRF
    try: os.mkdir(input + M + dirsuffix + os.sep + prog) 
    except:pass
    print "you chose ",prog,''	
    
    # MASKING genome with TRF if asked (default behavior) 
    # Create a unique genomeM masked File
    if not options.no_trf : 
        Mask.MaskAur(input, M+dirsuffix,prog)
    launch(input, prog)
	
#####################################################################
##########  -2.1-      IR SEARCH  
#####################################################################
def launch(input, prog):
    global dirsuffix,M
    # start time
    t0=time.time()
    genometoanalyse = input + M
    print genometoanalyse
    # launching prog for TIR search --> Result in BLAST-like format
    eval(prog)(genometoanalyse,prog)
    
    # filtering result  
    result=filter(input,prog)
    ########### STATISTICS #################	
    if result==None: 
        with open(genometoanalyse+'.dir/'+prog+'/statistic', 'a') as statistics:
            statistics.write('\nDone on :\t'+time.asctime()+'\nTotal time : \t' +str(time.time()-tinit)+'\n')
        return # NO CLUSTER AFTER FILTERING
    ########### STATISTICS #################	
    
    #sequence extraction
    extract(input,prog)


#####################################################################
##################  BEGIN IR SEARCH #################################
#####################################################################
def IRF(genome,prog):
	t0=time.time()
	optionIRF=' 2 3 5 80 10 40 '+str(max(10000 ,options.TIR_size[1]))+" "+str(options.MITE_size[1])
    # launch IRF
	print ">>> launching",options.program
	command_line = "irf "+ genome + optionIRF + ' -d -f -h'
	irf = subprocess.call(shlex.split(command_line))
	blastoutfile = ChangeIRFformat(genome, prog) # change .dat to IRF.out in BLAST format
	
    ########### STATISTICS #################	
	with open(genome+'.dir/'+prog+'/statistic', 'a') as statistics:
	    statistics.write(options.program+' processing :\t'+str(time.time()-t0)+' sec\n')
    ########### STATISTICS #################	
	return blastoutfile

def ChangeIRFformat(genome,prog):
    basename= genome+'.dir/IRF/'+os.path.basename(genome)
    outfile = genome + optionIRF.replace(' ','.') + '.dat' # as defined by IRF
    newoutfile = basename + optionIRF.replace(' ','.') +'.dat'
    os.rename(outfile , newoutfile) # move .dat file in IRF folder
    blastoutfile = SplitGenome.IRFBlastize(newoutfile) # change .dat to IRF.out in BLAST format
    return blastoutfile
    
def BLAST(genometosplit,prog):	
    t0=time.time()
    # split genome
    listsubcontig = SplitGenome.splitgenome(genometosplit,options.splitsize,dirsuffix) #### Genome split in 100000nt subsequences in DIR	  
    #listsubcontig = splitgenome(genometosplit,options.splitsize) #### Genome split in 100000nt subsequences in DIR	  
    t1= time.time()
    # launch BLAST
    print ">>> launching",options.program
    nbcpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
    progreturn=nbcpu.map(eval('SUB'+options.program), listsubcontig)    
    t2=time.time()
    # Concat result
    outfile=SplitGenome.concat(genometosplit,options.program,options.splitsize) # genome, prog, splitsize -->  'prog.out'
    #concatfile=concat(options.input,options.program) # genome, prog
    t3=time.time()
    
    with open(genometosplit+'.dir/'+options.program+'/statistic', 'a') as statistics:	
        statistics.write('Subcontig processing : '+ str(t1 - t0)+' sec\n')
        statistics.write('BLAST processing : '+ str(t2 - t1)+' sec\n')
        statistics.write('Concatenating result  : '+ str(t3 - t2)+' sec\n')
    
    return outfile
    
        
def SUBBLAST(newfile):	 # BLAST+ needed or invert comment line
	outfile=newfile+'.BLASTout'
	try:
	    command_line='blastn -task blastn-short -query %s -subject %s -out %s -word_size %s -outfmt 6 -dust no -evalue 50 '%(newfile,newfile,outfile,options.wordsize)
	    blast = subprocess.call(shlex.split(command_line))	   
	except:	
	    command_line='bl2seq -p blastn -i %s -j %s -o %s -W %s -D 1 -F F -e 50 '%(newfile,newfile,outfile,options.wordsize)
	    blast = subprocess.call(shlex.split(command_line))	   
	os.remove(newfile)
	
def IRFBLAST(genome,prog): pass
    
#####################################################################
##################  END IR SEARCH #################################
#####################################################################	

#####################################################################
##########  -2.2-      IR FILTER  
#####################################################################
def filter(input, prog): #  filtering sequences of prog.out file one by one --> return .allf file
	print '\n>>> filtering result'
	t0 = time.time()
	global dirsuffix,M
	
	inputfile = input+M+dirsuffix+'/'+prog+'/'+prog+'.out'
	allf =  input+M+dirsuffix+'/'+prog+'/'+prog+'.allf'
	option=(options.MITE_size,options.TIR_size,options.mismatch)
	
	res=filtreIR.BLASTfiltre(inputfile,allf,option) # 'prog.out'--> option filtered 'prog.allf'
	### STATISTICS ###
	t1=time.time()
	with open(input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
	    statistics.write('\nFiltering parameters:\n\tTIRs: '+repr(options.TIR_size)+'\n\tElement: '+repr(options.MITE_size)+'\n\tMismatch: '+str(options.mismatch)+'\n')
	    statistics.write('\nConcatenating and Filtering  lasted \t'+str(t1-t0)+' sec\n')
	    statistics.write('Filtering kept '+str(res[1])+' out of '+str(res[2])+'\n')
	### STATISTICS ###
	if res[1]==0 :return None
	
	if options.filter: extract(input,prog)
	elif options.launch: extract(input,prog)
	else: return res[1]
	
	
######################################################################
### -3.1- EXTRACT SEQUENCES AND FILTER FOR N AND LOW COMPLEXITY
######################################################################	
def extract(input, prog):
	global M, dirsuffix
	t0=time.time()
	# filtered Blast file to read
	inputMdir=input+M+dirsuffix
	file=open(inputMdir+'/'+prog+'/'+prog+'.allf').readlines()
	
	# Open files for list of hits to extract
	pba=open(inputMdir+'/'+prog+'/'+prog+'.allf.pba','w')
	itr=open(inputMdir+'/'+prog+'/'+prog+'.allf.pbaITR','w')
	
	for i,x in enumerate(file):
		if x=='':break
		pba.write(x.split('\t')[0]+'\t'+x.split('\t')[0]+'_'+x.split('\t')[6]+'_'+x.split('\t')[8]+'\t'+x.split('\t')[6]+'\t'+x.split('\t')[8]+'\t'+str(int(x.split('\t')[8])-int(x.split('\t')[6]))+'\n')
		itr.write(x.split('\t')[0]+'\t'+x.split('\t')[0]+'_'+x.split('\t')[6]+'_'+x.split('\t')[8]+'\t'+x.split('\t')[6]+'\t'+x.split('\t')[7]+'\t'+x.split('\t')[9].split('\n')[0]+'\t'+x.split('\t')[8]+'\t\n')
	pba.close()
	itr.close()

	print '\n>>> extracting sequences'
	b=0
	filewriteITR=open(inputMdir+'/'+prog+'/'+prog+'.allf.pbaITR.seq','w')
	filewrite=open(inputMdir+'/'+prog+'/'+prog+'.allf.pba.seq','w')
	filewriteF=open(inputMdir+'/'+prog+'/'+prog+'.allf.pba.seqfl', "w")

	fileitr=open(inputMdir+'/'+prog+'/'+prog+'.allf.pbaITR').readlines()		
	record_iterator = SeqIO.parse(input,'fasta')
	while True:
		try: supercontig = record_iterator.next()   
		except StopIteration: break	
		
		SeqITR = StringIO()
		Seq = StringIO()
		SeqFL = StringIO()
		for i,hit in enumerate(fileitr):
			if supercontig.id==hit.split('\t')[0]:
				minifl=max(int(hit.split('\t')[2])-60,0)
				average=(int(hit.split('\t')[5])+int(hit.split('\t')[2]))/2
				name=hit.split('\t')[1]
				seqitr=supercontig.seq[int(hit.split('\t')[2]):int(hit.split('\t')[3])+1]+supercontig.seq[int(hit.split('\t')[4]):int(hit.split('\t')[5])+1]
				seq=supercontig.seq[int(hit.split('\t')[2]):int(hit.split('\t')[5])+1]
				seqf=supercontig.seq[minifl:int(hit.split('\t')[5])+59]

				# filter against 'N', low complexity, and small
				flag=''
				for nt in ['A','T','G','C']:
					if float(seqitr.count(nt))/len(seqitr) > 0.8 or float(seqitr.count(nt))/len(seqitr) <0.1: flag='low'
				if seq.count('N')>10:flag='N'
				if int(hit.split('\t')[5])+1-int(hit.split('\t')[2])<80:flag='small'
				if flag=='':
					filewriteITR.write('>'+name+'\n'+str(seqitr)+'\n')#fasta_data1)
					filewrite.write('>'+name+'\n'+str(seq)+'\n')
					filewriteF.write('>'+name+'\n'+str(seqf)+'\n')
					b+=1
					
	filewriteITR.close()
	filewrite.close()
	filewriteF.close()

    ########### STATISTICS #################	
	print '\tFinal sequences number : ',b,' hits out of ',len(fileitr)			
	with open(inputMdir+'/'+prog+'/statistic', 'a') as statistics:
	    statistics.write('\nSequences extraction : '+str(time.time()-t0)+' sec\n')
	    statistics.write('Filtering for N and low complexity\n')
	    statistics.write('Sequences kept '+str(b)+' hits out of '+str(len(fileitr))+'\n')
    ########################################
	cluster(input,prog)
	
	
##############################################################################################
### -3.2-  CLUSTERIZE SEQUENCES, FILTER OUT CLUSTERS > X SEQUENCES, AND ALIGN CLUSTERS 
##############################################################################################
def cluster(input,prog):
	t0=time.time()
	fileseqfl=input+M+'.dir/'+prog+'/'+prog+'.allf.pba.seqfl'
	fileseq=input+M+'.dir/'+prog+'/'+prog+'.allf.pba.seq'
	
	try:os.mkdir(input+M+dirsuffix+'/'+prog+'/Temp')
	except:pass	
    
	# sort sequences without flanking by length
	print '\n>>> Sorting sequences by length and clustering'
	command_line3='uclust --quiet --sort %s --output %s.sorted' %(fileseq,fileseq)
	sort=subprocess.call(shlex.split(command_line3))	
	
	# cluster sequences without flanking'
	command_line4='uclust --quiet --cluster %s.sorted --uc %s.pba.seq.uc --id 0.70 --targetfract 0.5 --queryfract 0.5 --rev --log %s'%(fileseq,input+M+'.dir/'+prog+'/'+prog+'.allf', input +M+ '.dir/'+prog+'/uc.log')
	clust=subprocess.call(shlex.split(command_line4))
	
	# filtering for clusters > 2 sequences, and extracting sequences with flanking
	print '>>> Filtering clusters >=',options.minseq,' sequences  and extracting sequences'
	res=ucfAur.filtre(fileseq+'.uc',fileseq,os.sep.join([input+M+'.dir',prog,'Clusters']),options.minseq)
	t1=time.time()
	if res[1]==0: 
	    print "\n0 cluster found"
	    return
	# Aligning clusters > 2 sequences with MAFFT
	############################################################
	# CALL FUNCTION (doit) IN * multimafft.py *
	print '\n>>> Aligning clusters with flanking' # .seq --> .allafa
	nbcpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
	nbcpu.map(eval('multimafft.mafftor'), res[0]) #'*' list
	############################################################
	
    ### STATISTICS ###
	with open(input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
	    statistics.write('\nSequences clustering : '+str(t1-t0)+' sec\n')
	    statistics.write('Clusters alignment :'+str(time.time()-t1)+' sec\n')	
	    statistics.write(str(res[1])+ ' Clusters out of ' +str(res[2])+ ' \n\n')	
    ### STATISTICS ###

	removedup(input,prog)
	
	
####################################################################
####  -4-     REMOVE DUPLICATE SEQUENCES
####################################################################
# create .ifl.afa (aligned independant) 
def removedup(input,prog): # keep .seq (unalignd with flank)
    t0=time.time()
    global M, dirsuffix
    print '\n>>> Finding duplicates'
    
    fileseqfl=input+M+'.dir/'+prog+'/'+prog+'.allf.pba.seqfl'
    fileseq=input+M+'.dir/'+prog+'/'+prog+'.allf.pba.seq'
    
    ########### Sort seqfl file by length 
    command_line3='uclust --quiet --sort %s --output %s.sorted' %(fileseqfl,fileseqfl)
    sort=subprocess.call(shlex.split(command_line3))	
    
    # Extract flanking + 60 terminal nt  (separated 5' and 3')
    filefl=input+M+'.dir/'+prog+'/'+prog+'.allf.pba.fl'
    newlistfl=[]
    for sequence in SeqIO.parse(fileseqfl+'.sorted','fasta'):
        newlistfl.append(SeqRecord(seq=sequence.seq[:120],id=sequence.id+'-5', description=''))
        newlistfl.append(SeqRecord(seq=sequence.seq[-120:],id=sequence.id+'-3', description=''))
    SeqIO.write(newlistfl,filefl,'fasta')
    
    ########## Cluster identical flanking sequences 
    command_line4='uclust  --quiet --iddef 1 --cluster %s --seedsout %s.uniq --uc %s.uc --id 0.90 --rev  --log %s.log'%(filefl,filefl, filefl,filefl)
    clust=subprocess.call(shlex.split(command_line4))		
    
    # Remove duplicate in all.seq file (without flanking) 
    seqNewList=[]
    dup=[]
    seqfl = SeqIO.to_dict(SeqIO.parse(filefl+'.uniq', "fasta"))
    seqs = SeqIO.to_dict(SeqIO.parse(fileseq, "fasta"))
    for seq in seqs:
        if seq+'-5' in seqfl.keys() and seq+'-3' in seqfl.keys() : seqNewList.append(seqs[seq])
        else : dup.append(seq)
    # writing files with unique sequence without flanking and dup
    with open(fileseq+'.dup','w') as dupfile:
        dupfile.write('\n'.join(dup))
        
    SeqIO.write(seqNewList, fileseq+'.uniq', "fasta")

    if os.path.isdir(os.sep.join([input+M+'.dir',prog,'ClustersExclu1'])):
        pass#os.remove
    try: os.mkdir(os.sep.join([input+M+'.dir',prog,'ClustersExclu1'])) #
    except:pass
    
    ######################################################################
    # PARALLEL CALL FUNCTION (remove_duplicate) IN * remove_duplicate.py *
    fbasename=input+M+dirsuffix+'/'+prog+'/Clusters/*F.allafa'
    filetoclean=glob.glob(fbasename)
    m=len(filetoclean)
    bilan=open(input+M+dirsuffix+'/'+prog+'/ClustersBilanNbSeq','w')
    bilan.close()
    Results=options.cpu.map(eval('remove_duplicate.rmdup'), filetoclean)
    # return=1 for each cluster kept
    ######################################################################        

    ### STATISTICS ###            
    print '\t',len(seqNewList), 'unique sequences'
    print '\t',sum(Results) ,'clusters > 2 independant sequences out of ', m
    with open(input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
        statistics.write('\nDuplicate removing  :'+str(time.time()-t0)+' sec\n')	
        statistics.write(str(len(seqNewList))+  ' unique sequences\n')	
        statistics.write(str(sum(Results))+ ' Clusters \n')	
    ### STATISTICS ###
    
    findboundaries(input,prog)    
    
    
################################################################################   
##### -5-  FIND PRECISE BOUNDARIES BY COMPARING SEQUENCES AND REVERSE COMPLEMENT
################################################################################    
# # from .ifl.afa
# # create ifl.faa file for boundaries search
# # create .afa file independant, without flanking
# # create .fl file with flanking
# # create .fl53 file with flanking only 
def findboundaries(input,prog):
    t0=time.time()
    global M,dirsuffix
    print "\n>>> Finding boundaries"
    
    try:os.mkdir(input+M+dirsuffix+'/'+prog+'/ClustersExclu2')
    except:pass
    
    ###############################################################
    # PARALLEL CALL FUNCTION (createfiles) IN * FindBoundaries.py *
    Clustfiles=input+M+dirsuffix+'/'+prog+'/Clusters/*F'    
    filetoclean=glob.glob(Clustfiles+'.ifl.afa')
    options.cpu.map(eval('FindBoundaries.createfiles'), filetoclean)
    # for each cluster: create a file with sequences aligned with reverse complement 
    ###############################################################  

    ##################################################################
    # PARALLEL CALL FUNCTIONS (fb, clean, fb) IN * FindBoundaries.py *
    align=glob.glob(Clustfiles+'.ifl.faa')    
    Results=options.cpu.map(eval('FindBoundaries.AnalyseCluster'), align)
    # Return list of output files or 1 
    ##################################################################
    
    # Writing results (length of the TIRs in the alignment .afa)    
    TIR_File=open(input+M+dirsuffix+'/'+prog+'/Clusters/ListITR','w')
    n=0
    for result in Results: 
        if result==1: n+=1
        else: TIR_File.write(result)
    TIR_File.close()
    #clean
    align=glob.glob(Clustfiles+'.ifl.faa')    
    for file in align:
        os.rename(file,input+M+dirsuffix+'/'+prog+'/Temp/'+os.path.basename(file))
        shutil.move(file+'0',input+M+dirsuffix+'/'+prog+'/Temp/'+os.path.basename(file)+'0')  
    
    ### STATISTICS ###
    print '\t',len(Results)-n, 'clusters kept'
    with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
        statistics.write('\nTIR Boundaries search :'+str(time.time()-t0)+' sec\n')	
        statistics.write('Clusters kept :'+str(len(Results)-n)+'\n')
    ### STATISTICS ###
    
    RejectFalse(input,prog)
    
########################################################################    
###### -6-  REJECT CLUSTERS WITH TOO MANY GAPS
########################################################################  	
def RejectFalse(input,prog):
    t0=time.time()
    global M, dirsuffix
    
    try: os.mkdir(input+M+dirsuffix+os.sep+prog+os.sep+'ClustersExclu3/')
    except: pass
    
    print '\n>>> Checking for too many gaps'
    ##############################################################  
    # PARALLEL CALL FUNCTION (TooMuchGap) in * RejectClusters.py *
    FilesToCheck=glob.glob(input+M+dirsuffix+os.sep+prog+os.sep+'Clusters/*F.afa')
    Results=options.cpu.map(eval('RejectClusters.TooMuchGap'), FilesToCheck)
    ##############################################################  
    n=0
    for result in Results: 
        if result==1: n+=1

    ### STATISTICS ###
    t1=time.time()
    print '\t',n, ' clusters kept'
    with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
        statistics.write('\nChecking gaps :'+str(t1-t0)+' sec\n')	
        statistics.write('Clusters kept :'+str(n)+'\n')   
    ### STATISTICS ###
        

    MakeConsensus(input,prog)
    
########################################################################    
######## -7-  MAKE CONSENSUS FILE and extract TIRs
########################################################################
def MakeConsensus(input,prog):
	t0=time.time()
	print '\n>>> Making consensus'
	
	# read TIR list file for TIR length
	DicoTIR={}
	ITRpositions=open(input+M+'.dir/'+prog+'/Clusters/ListITR').readlines()
	for line in ITRpositions:
	   DicoTIR[line.split('\t')[0]]=(int(line.split('\t')[3]),int(line.split('\t')[4]))
	

	######################################################  
	# CALL FUNCTIONS (GenerateConsensus) IN * ConsAur.py *
	file = input+M+dirsuffix+'/'+prog+'/Clusters/*F.afa'	
	FileToCons = glob.glob(file)
	Results = options.cpu.map(eval('ConsAur.GenerateConsensus'),FileToCons)
	# RETURN A LIST OF CONSENSUS SEQUENCES
	######################################################   
	
	# WRITE COMPILATION TIR AND CONSENSUS files
	consfile=input+M+'.dir/'+prog+'/Clusters/Clust.Cons.fas'
	
	TIRfastafile=open(input+M+'.dir/'+prog+'/BilanTIR.fa','w')
	TIRfile=open(input+M+'.dir/'+prog+'/BilanTIR','w')	   
	consens=open(consfile,'w')		
	for TheCons in Results:
		# write consensus file "Clust.Cons.fas"
		consens.write('>'+TheCons.id+'\n'+str(TheCons.seq).replace('-','')+'\n')
		# write TIR consensus file "BilanTIR.fa"
		TIRfastafile.write('>'+TheCons.id+'\n'+str(TheCons.seq[:DicoTIR[TheCons.id][0]])+'\n')
		# write TIR compilation file "BilanTIR"
		TIRfile.write(TheCons.id+'\t'+str(len(TheCons.seq))
		+'\t'+str(len(TheCons.seq[:DicoTIR[TheCons.id][0]]))+'\t'+str(TheCons.seq[:DicoTIR[TheCons.id][0]])+'\t'
		+str(len(TheCons.seq[-DicoTIR[TheCons.id][1]:]))+'\t'+str(TheCons.seq[-DicoTIR[TheCons.id][1]:].reverse_complement())+'\n')
	consens.close()
	TIRfile.close()
	TIRfastafile.close()
	
	# CLUSTER CONSENSUS
	print '\n>>> Clustering consensus sequences'
	command_line='uclust --quiet --sort %s --output %s.sorted' %(consfile,consfile)
	sort=subprocess.call(shlex.split(command_line))	
	command_line2='uclust --quiet --cluster %s.sorted --seedsout %s.uniq --uc %s.uc --id 0.70 --rev '%(consfile,consfile,consfile)
	clust=subprocess.call(shlex.split(command_line2))
	
	res=ucfAur.filtre(consfile+'.uc',consfile,os.sep.join([input+M+'.dir',prog,'Cons']),'0')
	# clean cons directory
	for file in glob.glob(os.sep.join([input+M+'.dir',prog,'Cons/*'])):os.remove(file)
	ConsAur.clean(input+M+dirsuffix, prog, consfile)

	
	### STATISTICS ###
	with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
	    statistics.write('\nMaking consensus :'+str(time.time()-t0)+' sec\n')	
	    statistics.write('Consensus clustering : '+str(res[1])+'\n')	
	### STATISTICS ###
	
	FindTargetSite(input,prog)


########################################################################    
####### -8-  FIND TSD FROM EACH .fl53  FILE 
########################################################################
def FindTargetSite(input,prog):
    t0=time.time()    
    global M, dirsuffix
    FileToAnalyse=glob.glob(input+M+dirsuffix+'/'+prog+'/Cons/*F.fl53')
    print '\n>>> Finding TSD'
    
    ########################################################  
    # PARALLEL CALL FUNCTIONS (analyseTSD) IN * FindTSD.py *
    Results=options.cpu.map(eval('FindTSD.analyseTSD'), FileToAnalyse) # List of return
    ########################################################       

    ### WRITE COMPILATION TSD FILE
    with open(input+M+'.dir/'+prog+'/BilanTSD','w') as TSDFile: 
        TSDFile.write('\n'.join(Results))
    
    ### STATISTICS ###
    t1=time.time()
    print '\t',len(Results), ' clusters kept'
    with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
        statistics.write('\nFinding TSD :'+str(t1-t0)+' sec\n')	
    ### STATISTICS ###
    
    AutonomousSearch(input,prog)
	
	
########################################################################    
####### -9- SEARCH FOR LONGER ELEMENTS
########################################################################  	
def AutonomousSearch(input,prog):
	t0=time.time()
	print '\n>>> Searching longer complete elements'
	ret=Autonomous.autonomous(input+M+'.dir/'+prog+'/BilanTIR.fa', input)
	#if ret is not None:
	    #print '\n>>> Searching all longer copies'
	    #ret=Autonomous.incomplete(input+M+'.dir/'+prog+'/BilanTIR.fa', input)
	if ret is None: ret=0,0
    ### STATISTICS ###
	t1=time.time()
	with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
	    statistics.write('\nFinding autonomous :'+str(t1-t0)+' sec')	
	    statistics.write('\nTIR clusters: '+str(ret[0])+'\nUSEARCH clusters: '+str(ret[1])+'\n')	
    ### STATISTICS ###
    	
	GenerateListe(input,prog)
         
########################################################################    
########## -10- GENERATE  LIST OF MITES (LISTCOPIES.TXT)
########################################################################  	
def GenerateListe(input,prog):# generate list of MITEs
    print '\n>>> Generating MITE list'
    ListingCopy=open(input+M+'.dir/'+prog+'/BilanCopies.txt','w') 
    file=input+M+'.dir/'+prog+'/Cons/*F.allafa'
    align=glob.glob(file)
    #each file
    for clust in align:
        FileClust=open(clust).read().split('>')
        for seq in FileClust[1:]:
            supercontig=('_'.join(seq.split('_')[0:-2]))
            start=seq.split('_')[-2]
            end=seq.split('_')[-1].split('\n')[0]
            clustername=os.path.basename(clust).split('.')[0]
            ListingCopy.write(supercontig+'\t'+start+'\t'+end+'\t'+clustername+'\n')
            
    ListingCopy.close()

    print '\n>>> Generating Final File'
    DicoSeq={}
    
    Bilan=open(input+M+'.dir/'+prog+'/BilanNbSeq').readlines()
    for line in Bilan:
        if line.split('\t')[2].rstrip()!='1': DicoSeq[line.split('\t')[0]]=[line.split('\t')[0],line.split('\t')[1],line.split('\t')[2].rstrip()]
    ListTSD=open(input+M+'.dir/'+prog+'/BilanTSD').readlines()
    for line in ListTSD:
        if DicoSeq.has_key(line.split('\t')[0]): 
            DicoSeq[line.split('\t')[0]].append(max(line.split('\t')[1],line.split('\t')[4]))
            DicoSeq[line.split('\t')[0]].append('')    

    ListAutonomous=open(input+M+'.dir/'+prog+'/BilanAutonomous').readlines()
    for line in ListAutonomous:
        if DicoSeq.has_key(line.split('\t')[0]):
            list=DicoSeq[line.split('\t')[0]][:-1]+[line.split('\t')[1].rstrip()]
            DicoSeq[line.split('\t')[0]]=list
            
    ListTIR=open(input+M+'.dir/'+prog+'/BilanTIR').readlines()    
    for line in ListTIR:
        if DicoSeq.has_key(line.split('\t')[0]):
            ITRsize=[line.split('\t')[2],line.split('\t')[4]]
            ITR=[line.split('\t')[3],line.split('\t')[5].rstrip()]
            a=sorted(ITR,key=len)
            DicoSeq[line.split('\t')[0]].extend([ line.split('\t')[1] , max(ITRsize) , a[-1].upper() ])

    for key in DicoSeq.keys():
        if len(DicoSeq[key])==3: del DicoSeq[key]
        
    FinalFile=open(input+M+'.dir/'+prog+'/FinalBilan','w')
    FinalFile.write(('\t').join(['CLUSTERS','TOTAL','INDEPENDANT','TSD','AUTONOMOUS','LENGHT','TIR LENGTH','TIRSEQ'])+'\n')
    for clust in DicoSeq:
        FinalFile.write(('\t').join(DicoSeq[clust])+'\n')
    FinalFile.close()       

	### STATISTICS ###
    with open(options.input+M+'.dir/'+prog+'/statistic', 'a') as statistics:
        statistics.write('\nDone on :\t'+time.asctime()+'\nTotal time : \t' +str(time.time()-tinit)+'\n')
    print '\nDONE'
	### STATISTICS ###
	
    return
####################################    

####################################  		
def store_value(option, opt_str, value, parser):
    setattr(parser.values, option.dest, value)
    
####################################    
####################################    
####################################  


parser = OptionParser()
parser.set_defaults(program='IRF', splitsize=100000,TIR_size=(11,100),
          MITE_size=(80,750), mismatch=2, minseq=2, cpu=nbcpu)  
          
parser.add_option("-i", "--input", dest="input",help=" genome input file")
parser.add_option("-p", "--prog", dest="program",help=" program : BLAST for blastn,IRF for Inverted Tandem Repeat (default), PALIN for Palindrome")
parser.add_option("-s", "--splitsize", type='int',dest="splitsize",help=" splitsize [100000] + 5000")
parser.add_option("--TIRs", action='store', type='int', nargs=2, dest="TIR_size",help=" Min and Max TIR size (default 11 100)")
parser.add_option("--MITE", action='store', type='int', nargs=2, dest="MITE_size",help=" Min and Max MITE size (default 80 750)")
parser.add_option("-w", "--word", dest='wordsize',help=" Blast word size ")
parser.add_option("-m", "--mismatch", type='int',dest="mismatch",help=" Palindrome max mismatch allowed (default: 2)")
parser.add_option("--filter", action="store_true",dest="filter",help=" filtering of raw results")
parser.add_option("--concat", action="store_true",dest="concat",help=" concatenation of raw results")
parser.add_option("--cons", action="store_true",dest="cons",help=" consensus ")
parser.add_option("--extract", action="store_true",dest="extract",help="extraction of sequences")
parser.add_option("--cluster", action="store_true",dest="cluster",help=" clustering of sequences")
parser.add_option("--launch", action="store_true",dest="launch",help=" launching  IR search")
parser.add_option("--autonomous", action="store_true",dest="autonomous",help=" autonomous search")
parser.add_option("--TRF", action="store_true",dest="no_trf",help=" no trf sequence masking")
parser.add_option("--irfblast", action="store_true",dest="irfblast",help=" format irf file as blast")
parser.add_option("--rf",  type='str',dest="raw_filter",help=" ")
parser.add_option("--cpu",  type='int',dest="cpu",help="number of cpu to use")
parser.add_option("--minseq",  type='int',dest="minseq",help=" minimum number of sequences to keep a cluster")
(options,args) = parser.parse_args()
if options.TIR_size[1]<=20: options.mismatch=1
if not options.wordsize:options.wordsize=(max(4,(options.TIR_size[0]-options.mismatch)/options.mismatch))
if not options.cpu: options.cpu=multiprocessing.Pool(multiprocessing.cpu_count()) 
# creation of DIR directories and variables
dirsuffix='.dir'
if options.no_trf: M =''
else: M ='M'
    

    
if __name__ == '__main__':
    if not options.input: options.input=raw_input('Please provide a genome fasta file to analyse : ')

    try: os.mkdir(options.input + M + dirsuffix) 
    except:print str(options.input + M + dirsuffix), " already exists" 
    try: os.mkdir(options.input + M + dirsuffix + '/' +options.program) 
    except:print str(options.input + M +dirsuffix + '/' +options.program), " already exists\n"  
    
    # check for companion programs   
    for program in ['python', 'blastn', 'makeblastdb', 'blastall', 'formatdb', 'muscle', 'mafft', 'irf','trf', 'uclust']:
        if Check.cmd_exists(program)==False: print '****', program, 'does not seem to be installed or is not accessible'
        
    # create directory BLAST or IRF
    try: os.mkdir(options.input + M + dirsuffix + os.sep + prog) 
    except:pass
    
    # create options file
    with open(options.input + M + dirsuffix + '/' +options.program+'/options','w') as opt:
        opt.write('0_splitsize\t'+str(options.splitsize)+'\n1_minimum TIR size\t'+str(options.TIR_size[0]) +'\n2_maximum TIR size\t'+str(options.TIR_size[1])
        +'\n3_minimum MITE size\t'+str(options.MITE_size[0])+'\n4_maximum MITE size\t'+str(options.MITE_size[1])
        +'\n5_BLAST word size\t'+str(options.wordsize)+'\n6_number of mismatched allowed in TIR\t'+str(options.mismatch)+'\nminimum number of copies\t' + str(options.minseq) + '\n')

    ########### STATISTICS #################	
    with open(options.input + M + dirsuffix + '/'+options.program +'/statistic','a') as statistics:
        statistics.write('\n\n###### NEW ANALYSIS ON '+time.asctime()+'######\nFasta file :\t'+ options.input +'\n'+options.program+'\n')
        statistics.write(str(options)+'\n')
    ########### STATISTICS #################	
    
    # skip begining
    if options.concat: concat(options.input,options.program)        
    elif options.filter: filter(options.input,options.program)         
    elif options.extract: extract(options.input,options.program)         
    elif options.cluster: cluster(options.input,options.program)          
    elif options.launch: launch(options.input,options.program)          
    elif options.autonomous: AutonomousSearch(options.input,options.program)          
    elif options.cons: MakeConsensus(options.input,options.program)          
    elif options.irfblast: ChangeIRFformat(options.input,options.program)          
    #elif options.pals: PALSfiltre.PALS(options.input,dirsuffix)         
    else: goon(options.input, options.program)
#else:
    #print "Don't use this fucking interpreter!"
        

