#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################
import re, os, sys, shutil
import AnalyseIR

# filter blast-like results (from BLAST or IRF)
def BLASTfiltre(inputfile,allf,option): 

	# option = (options.MITE_size, options.TIR_size, options.mismatch)
	dicontig={}
	filteredhsp=0
	tothsp=0
	resultfiltre=open(allf,'wb')
	resultfiltre2=open(allf+'1','wb')
	
	result=open(inputfile).readlines()
	for hsp in result:
		if hsp.split('\t')[0] not in dicontig.keys() :dicontig[hsp.split('\t')[0]]=[] #new supercontig dict
		if int(hsp.split('\t')[8])<int(hsp.split('\t')[9]):tothsp+=1# if bad orientation of the second TIR
		elif abs(int(hsp.split('\t')[7])-int(hsp.split('\t')[8]))>option[0][1]:tothsp+=1# if > max size (internal sequence)
		elif abs(int(hsp.split('\t')[6])-int(hsp.split('\t')[9]))<option[0][0]:tothsp+=1# if < min size
		elif int(hsp.split('\t')[7])>int(hsp.split('\t')[8]):tothsp+=1# if overlapping TIRs
		elif int(hsp.split('\t')[3])>option[1][1] or int(hsp.split('\t')[3])<option[1][0]:tothsp+=1# min TIR or max TIR
		elif 'BLAST' in inputfile and int(float(hsp.split('\t')[4]))>option[2]:tothsp+=1# mismatch for BLAST
		# elif 'IRF' in inputfile  and abs(int(hsp.split('\t')[6])-int(hsp.split('\t')[9]))-(2*int(hsp.split('\t')[3]))<10:tothsp+=1
		else: 
		    if hsp.split('\t') not in dicontig[hsp.split('\t')[0]]: # new hsp (not in overlappinp subcontig)
		        dicontig[hsp.split('\t')[0]].append(hsp.split('\t')) # list of list
		        open(allf+'1','a').write(hsp)
		        filteredhsp+=1
		        tothsp+=1
	######## overlapping hit
	newdict=[]
	for supercontig in dicontig:
		n=0

		sorteddicontig=sorted(dicontig[supercontig], key=lambda start: int(start[6])) # sort list of hsp by start query
		for i,hsp in enumerate(sorteddicontig) :
		    if i!=0 and int(hsp[9]) < int(sorteddicontig[i-1][9]): pass # shorter element : eliminated
		    elif i!=0 and int(hsp[6]) < int(sorteddicontig[i-1][9]): # overlapping element, take a large one
		        hsp.pop(6)
		        hsp.insert(6,sorteddicontig[i-1][6])
		        newdict.pop()
		        newdict.append(('\t').join(hsp))
		    else: 
		        newdict.append(('\t').join(hsp))
		        n+=1
		        
	for hit in newdict:
	    open(allf,'a').write(hit)
	print '\tnumber of hsp MITE found :',len(newdict), 'out of ', tothsp, filteredhsp
	
	# Sorting Blastfile
	os.system('sort '+allf+' -o '+allf+'  -k 1,2 -k 7n ')
	os.system('sort '+allf+'1 -o '+allf+'1  -k 1,2 -k 7n ')
	   
	dicontig.clear()
	return allf, filteredhsp,tothsp	


# not used anymore
def IRFfiltre(genome , outfile, optionIRF): 
	#genome=IrfResultFile.split('.2.3.5.80.10.40.10000.750.dat')[0]
	print '>>> processing IRF results'
	#shutil.copyfile(genome, genome+'.save')
	file=open(outfile).read().split('Sequence: ')[1:]

	liste=[]
	listemp=[]
	for contig in file:
	    palindromef=[]
	    listemp=[]
	    #print contig.split('\n')[0]
	    palindrome=contig.split('\n')[7:-3]
	    for i,line in enumerate(palindrome):
	        if int(line.split(' ')[2])<10 or  int(line.split(' ')[2])>100:pass
	        elif int(line.split(' ')[6])<10 or int(line.split(' ')[6])>750:pass 
	        elif float(line.split(' ')[12])>90 or float(line.split(' ')[13])>90:pass 
	        else: palindromef.append((int(line.split(' ')[0]),line))
	    #sorted(palindromef, key=lambda pal: pal[0])	
	    for i,line in enumerate(sorted(palindromef, key=lambda pal: pal[0])):
	        hsp=(contig.split('\n')[0],int(line[1].split(' ')[0]),int(line[1].split(' ')[1]),int(line[1].split(' ')[4]),int(line[1].split(' ')[3]), int(int(line[1].split(' ')[2])*float(line[1].split(' ')[7])/100),int(line[1].split(' ')[2]))
	        if i==0: 
	            listemp.append(hsp)
	        #chevauchant
	        elif line[0]<int(listemp[-1][4]):
	            listemp.append(hsp)
	        #non chevauchant
	        else:
	            liste+=AnalyseIR.templistanalyse(listemp)
	            listemp=[]
	            listemp.append(hsp)
            # si dernier
	        if i==len(palindromef)-1:
	            liste+=AnalyseIR.templistanalyse(listemp)     
	            #print tmplist                    
	            listemp=[]
	listemp=[]
	
    # output file creation BLAST format
	allf=open(genome+'.dir/IRF/IRF.allf','w')
	for x in liste:
	    allf.write(x[0] + '\t' + x[0] +'\t\t'+str(int(x[2])-int(x[1]))+'\t'+str(x[5])+'\t\t'+str(x[1])+'\t'+str(x[2])+'\t'+str(x[3])+'\t'+str(x[4])+'\t\n')
	allf.close()
	#os.rename(genome + optionIRF + '.dat',genome+'.dir/IRF/'+os.path.basename(genome) + optionIRF +'.dat')
	shutil.copyfile(genome+'.dir/IRF/'+os.path.basename(genome)+ optionIRF + '.dat',genome+'.dir'+'/IRF/'+os.path.basename(genome)+'.IRFout')
	return len(liste)

    
# Compare all positions within a supercontig to eliminate redundant and overlaping elements
def ComparePosition(nom,list5s,list5e,list3s,list3e,listmatch):
        returnedData=[]

        # keep hits that define MITEs of good length    
        # keep hits that present at max 10% mismatch in TIRs        
        lf=[] # filtered list

        for i,j in enumerate(list5s): 
            mmatch=str(int(list5e[i])-int(list5s[i])+1-listmatch[i])
            lenmatch=str(int(list3e[i])-int(list5s[i])+1)
            if 80<=(int(list3e[i])-int(list5s[i])+1)<=750 and int(mmatch)<=float(int(list5e[i])-int(list5s[i])+1)/10:
               #print int(mmatch),float(int(list5e[i])-int(list5s[i])+1)/10
                hsp= (nom,int(list5s[i]),int(list5e[i]),int(list3e[i]),int(list3s[i]),mmatch,lenmatch) #tuple with integers
                lf.append(hsp)

        # put overlapping sequences in blocklist for further analysis   
        # put non overlapping sequences i                 
        blocklist=[]
        # check if nested or untangled hits among successive candidates hits
        for i,j in enumerate(lf):    
            # first hit saved in block
            if i==0:
                blocklist.append(j)  
                
            # overlapping hit --> saved in block
            # 	<====----====> i-1						----====>    i-1						<====----
            #	..........<====---- i					----====>    i							<====----
            elif  lf[i][1]<=lf[i-1][3] or lf[i][3]<= lf[i-1][3] or lf[i][1]== lf[i-1][1]: #enchevetre
                blocklist.append(j)
            # non overlapping 
            elif lf[i][1]>lf[i-1][3]: 
                # --> analysis of previous block, block chosen hit saved in liste
                returnedData.append(AnalyseIR.noanalyse(blocklist))     
                
                # --> hit saved in new empty block
                blocklist=[]
                blocklist.append(j)
                
            # last hit --> check if nested or untangled, block chosen hit saved in liste
            if i==lf.index(lf[-1]):
                returnedData.append(AnalyseIR.noanalyse(blocklist))    
                blocklist=[]
                
        return returnedData