#!usr/bin/env python

#################  AutoMITAur.v1.0: A tool for de novo detection  of MITE ##################
#
# Main objective: de novo detection of MITE in sequenced genome
#
# Scientific contributors:  Aurelie Hua-Van
#
#
# Copyright 2013 Aurelie Hua-Van, ahuavan@legs.cnrs-gif.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
## SCIENTISTS: please be aware that the fact that this program is released
## as Free Software does not excuse you from scientific propriety, which
## obligates you to give appropriate credit! If you write a scientific paper
## describing research that made substantive use of this program, it is your
## obligation as a scientist to (a) mention the fashion in which this software
## was used, including the version number, with a citation to the literature,
## in the Methods section, to allow replication; and/or (b) mention this
## software in the Acknowledgements section.
#
#############################################################################

import re, sys, shutil, os, subprocess, shlex, time, glob
from Bio import SeqIO


dico_Seq={}
dico_SeqF={}
clusternb=[]
clusternbname={}
clusternbseq={}
clusternbseqF={}

def ask():
	
	question = raw_input('Uclust file ? ').rstrip()
	seqfile = raw_input('Sequences file ? ').rstrip()
	filePath =  raw_input('Output directory? ').rstrip()	
	if os.path.isdir(filePath):
	    answer=raw_input('%s already exist\nDo you really want to delete directory ? [N]/Y'%filePath)
	    if answer=='Y' or answer=='y' : shutil.rmtree(filePath)
	    else: filePath =  raw_input('Repertoire de sortie ? ').rstrip()	

	keep = raw_input('Keep clusters with at least [2] sequences only ?')
	
	if keep=='': keep='2'
	if filePath=='':filePath= os.sep.join([os.path.dirname(question),'BLAST/Clusters'])
	filtre(question,seqfile,filePath,keep)


def filtre(uc,seqfile,filePath,keep):

	#read uc
	file=open(uc).readlines()

	try: shutil.rmtree(filePath)	
	except:pass

	try: 
	    os.mkdir(filePath+'Exclu0')	
	except:pass

	os.mkdir(filePath)
	
	dico_Seq.clear()
	dico_SeqF.clear()
	clusternbname.clear()
	clusternb=[]
	clusternb0=[] # list of cluster t reject
	clusternbseq.clear()
	clusternbseqF.clear()	
	exclu=[]

	# selecting cluster with at least x sequences, save seed sequence as in dicos entru
	for line in file:
		if line.split('\t')[0]=='#' or line=='\n':pass
		elif line.split('\t')[0]=='C' : 
		    if int(line.split('\t')[2]) < int(keep) and int(line.split('\t')[1]) not in clusternb0: 
		        clusternb0.append(int(line.split('\t')[1])) # create entry for cluster (N0) to reject
		        exclu.append(line.split('\t')[8])
		    elif int(line.split('\t')[2]) >= int(keep) and int(line.split('\t')[1]) not in clusternb:
		        clusternb.append(int(line.split('\t')[1]))# create entry for cluster (N0) to keep
		    #dico des cluster pour fichier de sequence
		    clusternbname[int(line.split('\t')[1])]='' #dico[NoClust]=
		    clusternbseq[int(line.split('\t')[1])]=''
		    clusternbseqF[int(line.split('\t')[1])]=''

	clusternb.sort()
	print '\t',len(clusternb0)+ len(clusternb), 'clusters in total'
	print '\t',len(clusternb), "clusters with at least",keep, "sequences"
	if len(clusternb)==0: return None,0
	
	# check for flanking sequence file
	if os.path.isfile(seqfile+'fl'): pass
	else: 
		shutil.copyfile(seqfile, seqfile+'fl') 
		print 'CAREFUL, no file with flanking were found'
	# hash table with seq from Allseq flanking file
	for seq_record in SeqIO.parse(seqfile+'fl', "fasta"):
		dico_SeqF[seq_record.id]=seq_record.seq
	
	#extracting all sequences belonging to cluster >=keep  from uc file
	for line in file:
		if line.split('\t')[0]in'HS': 
			# retournement if needed
			if line.split('\t')[4]=='+' or line.split('\t')[4]=='*':# sens +
				#LaSeq=dico_Seq[line.split('\t')[8]]
				LaSeqFlank=dico_SeqF[line.split('\t')[8]]
				#clusternbseq[int(line.split('\t')[1])]+='>'+line.split('\t')[8]+'\n'+LaSeq.tostring()+'\n'
				clusternbseqF[int(line.split('\t')[1])]+='>'+line.split('\t')[8]+'\n'+LaSeqFlank.tostring()+'\n'
				
			elif line.split('\t')[4]=='-':# sens -
				#LaSeq=dico_Seq[line.split('\t')[8]]
				#LaSeqR=LaSeq.reverse_complement()
				#dico_Seq[line.split('\t')[8]]=LaSeq
				
				LaSeqFlank=dico_SeqF[line.split('\t')[8]]
				LaSeqFlankR=LaSeqFlank.reverse_complement()
				dico_SeqF[line.split('\t')[8]]=LaSeqFlankR
				
				#clusternbseq[int(line.split('\t')[1])]+='>'+line.split('\t')[8]+'\n'+LaSeqR.tostring()+'\n'
				clusternbseqF[int(line.split('\t')[1])]+='>'+line.split('\t')[8]+'\n'+LaSeqFlankR.tostring()+'\n'			

			clusternbname[int(line.split('\t')[1])]+=line.split('\t')[8]+line.split('\t')[4]+'\t'

	ClustBil=open(uc+'.bilan','w')
	ClustBil.write(file[0])	
	
	#writing exclu file
	ExcluSeq=open(filePath+'Exclu0/Exclu.seq','w')
	for line in exclu:
		ExcluSeq.write('>'+line+'\n'+(dico_SeqF[line]).tostring()+'\n')
		ExcluSeq.write('>'+line+'\n'+(dico_SeqF[line]).reverse_complement().tostring()+'\n')
		
	for clust in clusternb0:
		cluster=open(filePath+'Exclu0/ClustF%sF.seq'%str(clust),'w')
		cluster.write(clusternbseqF[clust])
		cluster.close()		
		
	for clust in clusternb:
		cluster=open(filePath+'/ClustF%sF.seq'%str(clust),'w')
		cluster.write(clusternbseqF[clust])
		cluster.close()		
		ClustBil.write(str(clust)+'\t'+clusternbname[clust].rstrip()+'\n')
		
	ExcluSeq.close()
	ClustBil.close()
	return glob.glob(filePath+'/ClustF*F.seq'), len(clusternb),len(clusternb0)+ len(clusternb)
   
	
	

if __name__ == '__main__' : 

    if len(sys.argv)==5:filtre(sys.argv[1],sys.argv[2],sys.argv[3],sys.argv[4])
    
    else: ask()
else: 
    pass
